/*
 * Flash utility for HW-A03
 *
 * Copyright (C) 2016 SHARP.Co.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * updater のロード
 * <pre>
 *  md5の処理を行う
 * </pre>
 * @version 0.01
*/

#include "updater.h"
#include "rom_image.h"
#include "md5.h"
#include "common.h"
#include <spi.h>
#include <spi_flash.h>

extern int do_qspi(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[]);

struct spi_flash *qspi_flash = NULL;

/* Flashの初期化 */
void updater_spiflash_init(void)
{
	if (qspi_flash == NULL) {
		DBG_PRINT("flash init");
		qspi_flash = spi_flash_probe(CONFIG_SF_DEFAULT_BUS, CONFIG_SF_DEFAULT_CS, 
								CONFIG_SF_DEFAULT_SPEED, CONFIG_SF_DEFAULT_MODE);
	}
}

/*  read */
int	updater_rom_read(u8 *address, u8 *dest, int size)
{
	int ret = 0;
	memcpy(dest, address, size);

	return ret;
}

int	updater_rom_state(int state)
{
	int ret = 0;
	return ret;
}

/* erase */
int	updater_rom_erase(u32 addr, u32 size)
{
	int ret = 0;
	u32 offset = addr;

	offset -= CONFIG_SYS_TEXT_BASE;
	updater_spiflash_init();
	size = qspi_flash->erase_size;
	ret = spi_flash_erase(qspi_flash, offset, size);

	return ret;
}

/* QSPIモードに変更 */
int updater_rom_initial_retry(void)
{
/* sf probe および QSPIモードを変更 qspi singleコマンドの実行 */
	int ret = 0;
	static int is_first = 1;

	char *argv[2] = {"qspi", "single"};
	if (is_first) {
		ret = do_qspi(NULL, 1, 2, argv);
		is_first = 0;
	}

	return ret;
}

int updater_rom_deinitial_retry(void)
{
	int ret = 0;
	return ret;
}

int updater_rom_erase_check(u32 address,int size)
{
	int ret = 0;

	return ret;
}

int	updater_rom_write_no_erase(u8 *address, u8 *source, int size)
{
	int ret = 0;
	u32 offset = (u32)address;

	offset -= CONFIG_SYS_TEXT_BASE;
	ret = spi_flash_write(qspi_flash, offset, size, source);

	return ret;
}

int	updater_rom_write(u8 *address, u8 *source, int size)
{
	int	result = 0;
	u32 addr = (u32)address;

	result = updater_rom_erase(addr, size);
	if (result != 0) {
		goto END;
	}

	result = updater_rom_write_no_erase(address, source, size);

END:
	return result;
}

