/*
 * Updater for HW-A03
 *
 * Copyright (C) 2016 SHARP.Co.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/*
 * UPDATER Header
*/
#ifndef	__UPDATER_H__
#define	__UPDATER_H__

#include "ffs.h"
#include "dct.h"
#include "keypassver.h"
//#define UPDATER_DEBUG
#ifdef UPDATER_DEBUG
#define DBG_PRINT(...)				{printf("%s(%d):", __func__, __LINE__); \
									 printf(__VA_ARGS__);}
#else /* UPDATER_DEBUG */
#define DBG_PRINT(...)				((void)0)
#endif /* UPDATER_DEBUG */

#define ENABLE_HEADER_DUMP

#define	QSPI_ROM_STATE_PROTECTED	(0)
#define	QSPI_ROM_STATE_UNPROTECT	(1)

#define UPDATER_DT_OFFSET			(0x180C0000)
#define UPDATER_DCT1_OFFSET			(0x18140000)
#define UPDATER_DCT2_OFFSET			(0x18180000)
#define UPDATER_KERNEL_OFFSET		(0x18200000)
#define UPDATER_RFS_OFFSET			(0x18600000)
#define UPDATER_EXSYSTEM_OFFSET		(0x19000000)
#define UPDATER_FIRM_OFFSET			(0x19100000)

#define UPDATER_LOAD_ADDR			(0x20040000)

#define UPDATER_DT_SIZE				(1024*256)			/* 256KB */
#define UPDATER_DCT_SIZE			(1024*64)
#define UPDATER_DCT1_SIZE			UPDATER_DCT_SIZE	/*  64KB */
#define UPDATER_DCT2_SIZE			UPDATER_DCT_SIZE	/*  64KB */
#define UPDATER_KERNEL_SIZE			(1024*1024*4)		/*   4MB */
#define UPDATER_RFS_SIZE			(1024*1024*10)		/*  10MB */
#define UPDATER_EXSYSTEM_SIZE		(1024*1024*1)		/*   1MB */
#define UPDATER_FIRM_SIZE			(1024*1024*15)		/*  15MB */

#define	UPDATER_RETRY_LIMIT			(3)

#define	UPDATER_IV_SIZE				(16)				// 乱数サイズ(128Bit)
#define	UPDATER_KEY_SIZE			(16)				// 鍵通知サイズ(128Bit)
#define	UPDATER_BLOCK_SIZE			FFS_BLOCK_SIZE		// データバッファサイズ

#define	UPDATER_TRUE				(1)
#define	UPDATER_FALSE				(0)

#define UPDATER_QSPI_RESET_TIMEOUT	(5000)				// QSPI再起動タイムアウト(強制再起動)
#define UPDATER_QSPI_INIT_TIMEOUT	(2000)				// QSPI初期化タイムアウト(retry処理)
#define UPDATER_QSPI_INIT_DELAY		(100)				// QSPI初期化ディレイ

#define UPDATER_LED_BRK_TIMEOUT		(500)				// LED、Break信号処理用タイムアウト
#define UPDATER_LED_TOGGLE_TIME		(500)				// LED信号トグル間隔
#define UPDATER_BRK_TOGGLE_TIME		(1000)				// Break信号トグル間隔

#define UPDATER_LED_SW_CONNECTED	(1)					// LED/SWボード接続状態

#define LED_ON	(0)
#define LED_OFF	(1)

#define SW_ON	(0)
#define SW_OFF	(1)

#define BRK_ON	(0)
#define BRK_OFF	(1)

#define FORCE_UPDATER	(0x00000001)
#define FORCE_RESCUE	(0x00000002)
#define FORCE_HALT		(0x00000004)


#define swap16(x) 	((x >> 8) & 0x00ff) | ((x << 8) & 0xff00)
#define swap32(x) 	(((x & 0x000000ffL) << 24) | \
					 ((x & 0x0000ff00L) <<  8) | \
					 ((x & 0x00ff0000L) >>  8) | \
					 ((x & 0xff000000L) >> 24))

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define be16toh(x)	swap16(x)
#define be32toh(x)	swap32(x)
#define htobe16(x)	swap16(x)
#define htobe32(x)	swap32(x)
#define le16toh(x)	(x)
#define le32toh(x)	(x)
#define htole16(x)	(x)
#define htole32(x)	(x)
#elif __BYTE_ORDER == __BIG_ENDIAN
#define be16toh(x)	(x)
#define be32toh(x)	(x)
#define htobe16(x)	(x)
#define htobe32(x)	(x)
#define le16toh(x)	swap16(x)
#define le32toh(x)	swap32(x)
#define htole16(x)	swap16(x)
#define htole32(x)	swap32(x)
#endif

extern u8 updater_read_buffer[UPDATER_BLOCK_SIZE];
extern u8 updater_write_buffer[UPDATER_BLOCK_SIZE];

// Code ROM Read/Write
extern void updater_spiflash_init(void);
extern int updater_rom_initial(void);
extern int updater_rom_initial_retry(void);
extern int updater_rom_deinitial(void);
extern int updater_rom_deinitial_retry(void);
extern int updater_rom_erase(u32 addr, u32 size);
extern int updater_rom_state(int state);
extern int updater_rom_write(u8 *address, u8 *source, int size);
extern int updater_rom_write_no_erase(u8 *address, u8 *source, int size);
extern int updater_rom_read(u8 *address, u8 *dest, int size);
extern int updater_rom_erase_check(u32 address, int size);

// MD5
int updater_md5_hash(u8 *hash, u8 *data, int size);
int updater_md5_hash_check(u8 *hash, u8 *comp);

// MTU2
extern void updater_mtu2_Init (void);
extern void updater_mtu2_delay(int msec);

// DCT
extern u32 updater_dct_get_active(void);
extern int updater_dct_data_update(u8 *data);
extern int updater_dct_header_read(dct_header_t *dct_hdr);
extern int updater_dct_header_write(dct_header_t *dct_hdr);
extern int updater_dct_retry_read(u8 *retry);
extern int updater_dct_retry_write(u8 *retry);
extern int updater_dct_state_read(u8 *state);
extern int updater_dct_state_write(u8 *state);
extern int updater_initial_dct(void);
extern int updater_image_write(u8 *woffset, u32 size, u8 *md5_hash);
extern START_MODE get_startmode(void);
extern void set_startmode(START_MODE mode);

// FFS
extern int updater_ffs_read_init(u32 read_start);
extern int updater_ffs_read(u8 *data, u32 size);
extern int updater_ffs_read_block(u8 *data, int top);


// LED SW
extern void power_led(u8 on);
extern void wlan_led(u8 on);
extern void link_led(u8 on);
extern u8 wlan_sw(void);
extern u8 link_sw(void);
extern void enable_uart_brk_pin(void);
extern void brk_send(u8 on);
extern void led_flush(void);
extern u8 led_sw_board(void);

// updater
extern int load_updater(u32 loadaddr);

#ifdef UPDATER_DEBUG
// for debug
extern void updater_dump(u8 *data, int length);
#endif /* UPDATER_DEBUG */

#endif	/* __UPDATER_H__ */
