/*
 * Updater for HW-A03
 *
 * Copyright (C) 2016 SHARP.Co.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * updater のロード
 * <pre>
 *  md5の処理を行う
 * </pre>
 * @version 0.01
*/

#include "updater.h"
#include "rom_image.h"
#include "md5.h"
#include "aes.h"

u8 enc_key[16] = ENC_KEY;	/* 復号キー   */
u8 enc_iv[16]  = ENC_IV;	/* 初期ベクタ */

u8 updater_read_buffer[UPDATER_BLOCK_SIZE];					// データ
u8 updater_write_buffer[UPDATER_BLOCK_SIZE];				// データ

int updater_image_load(u8 *roffset, u8 *woffset, u32 size, u8 *md5_hash)
{
	int rc;
	int ret = -1;
	int first_flag = UPDATER_FALSE;
	u8 md5_comp[MD5_HASH_SIZE];
	u8 *raddr;
	u8 *waddr;
	u32 rsize = 0;
	u32 wsize = 0;

	DBG_PRINT("roffset  = %p\n", roffset);
	DBG_PRINT("woffset  = %p\n", woffset);
	DBG_PRINT("size     = %d\n", size);

	/* save data */
	raddr = roffset;
	waddr = woffset;
	wsize = size;

	/* Init QSPI ROM(SPI mode & unprotect) */
	rc = updater_rom_initial_retry();
	if (rc != 0) {
		printf("updater_rom_initial_retry error\n");
		goto END;
	}

	/* write */
	DBG_PRINT("write start\n");
	raddr = roffset;
	while (wsize > 0) {
		if (wsize <= UPDATER_BLOCK_SIZE) {
			rsize = wsize;
		} else {
			rsize = UPDATER_BLOCK_SIZE;
		}
		DBG_PRINT("raddr = %p\n", raddr);
		DBG_PRINT("waddr = %p\n", waddr);
		DBG_PRINT("rsize = %d\n", rsize);

	    rc = updater_ffs_read(updater_read_buffer, rsize);
		if (rc != 0) {
			printf("updater_ffs_read error: size=%d\n", rsize);
			goto END;
		}

		if (first_flag == UPDATER_FALSE) {
			AES128_CBC_decrypt_buffer(waddr, updater_read_buffer, 
										rsize, enc_key, enc_iv);
			first_flag = UPDATER_TRUE;
		} else {
			AES128_CBC_decrypt_buffer(waddr, updater_read_buffer, 
										rsize, 0, 0);
		}

		wsize -= rsize;
		raddr += rsize;
		waddr += rsize;
		printf("%02d%% write\n", ((size - wsize)*100) / size);
	}
	printf("\n");
	DBG_PRINT("write end\n");

	/* check md5sum */
	DBG_PRINT("check md5sum start\n");
	updater_md5_hash(md5_comp, woffset, size);
	rc = updater_md5_hash_check(md5_comp, md5_hash);
	if (rc != UPDATER_TRUE) {
		printf("md5sum error: addr=0x%x size=%d\n", (u32)woffset, size);
		goto END;
	}
	DBG_PRINT("check md5sum end\n");

	ret = 0;
END:
	return ret;
}

/**
 * @brief アップデータのロード
 * @attention なし
 * @note 
 * @param [IN] loadaddr
 * @return OK/ERROR
 */
int load_updater(u32 loadaddr)
{
	int rc;
	int ret = -1;
	u8 *output;		/* アップデータ展開先(復号後) */
	u8 *input;		/* アップデータ暗号 */

	linux_image_header_t linux_img_hdr;

	/* Flashの初期設定を実施 */
	rc = updater_rom_initial_retry();

	if (rc != 0) {
		printf("flash error: flash init error\n");
		goto END;
	}

	/* ROMイメージヘッダ情報を取得する */
		/* 必要な情報はupdaterのサイズと、updater復号後のmd5sum値 */
	/* read Linux CPU image header */

    rc = updater_ffs_read_init(UPDATER_FIRM_OFFSET);
	if (rc != 0) {
		printf("updater_ffs_read_init error: addr=%08x\n", UPDATER_FIRM_OFFSET);
		goto END;
	}

	rc = updater_ffs_read((u8 *)&linux_img_hdr, sizeof (linux_image_header_t));
	if (rc != 0) {
		printf("read error: linux image header\n");
		goto END;
	}

	/* check chunk type */
	if (strncmp((char*)linux_img_hdr.chunk_type, (char*)UPDATE_HEADER_CHUNK_LINUX_CPU_IMAGE,
			(sizeof (linux_img_hdr.chunk_type))) != 0) {

		printf("error: linux image header chunk\n");
		goto END;
	}

	/* ROMイメージデータの先頭(updaterのデータ格納)からloadaddrで展開する */
	output = (u8 *)loadaddr;

	input = (u8 *)(UPDATER_FIRM_OFFSET + sizeof (linux_image_header_t));

	rc = updater_image_load(input, output, be32toh(linux_img_hdr.updater_size), linux_img_hdr.updater_hash);

	/* OK or NGの判断 */
	if (rc != 0) {
		printf("error: updater load error\n");
		goto END;
	}
	ret = 0;
END:
	return ret;
}

