/*
 * Updater for HW-A03
 *
 * Copyright (C) 2016 SHARP.Co.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

/**
 * updater のロード
 * <pre>
 *  md5の処理を行う
 * </pre>
 * @version 0.01
*/

#include "rescue.h"
#include "rom_image.h"
#include "md5.h"
#include "common.h"
#include <xyzModem.h>

static connection_info_t info;
static int open_flg = 0;

static int getcxmodem(void) {
	if (tstc())
		return (getc());
	return -1;
}

/* open ymodeのストリームをクローズする */
static void load_serial_ymode_close(bool flag)
{
	int err;

	if (open_flg != 0) {
		xyzModem_stream_close(&err);
		xyzModem_stream_terminate(flag, &getcxmodem);
		open_flg = 0;
	}
}

/* open ymodeのストリームをオープンする */
static int load_serial_ymode_open(void)
{
	int err;
	int res;
	int ret = 0;

	if (open_flg) {
		/* すでにopen済みならopenしない */
		goto end;
	}

	info.mode = xyzModem_ymodem;
	res = xyzModem_stream_open(&info, &err);
	if (res != 0) {
		ret = -1;
		printf("%s\n", xyzModem_error(err));
		goto end;
	}
	ret = 0;
	open_flg = 1;
end:
	return ret;

}

static int load_serial_ymodem(u8* roffset, int size)
{
	int remain = size;
	int err;
	int res = 0;

	while(remain > 0) {
		res = xyzModem_stream_read((char*)roffset, YBUFFER, &err);

		/* 読み込むデータがない場合は抜ける */
		if (res <= 0) {
			break;
		}
		led_flush();
		remain -= res;
		roffset += res;
	}

	return remain;
}

/* ymodemロードの初期化を行う */
int rescue_init(void)
{
	return load_serial_ymode_open();
}

/* ymodemロードの終了処理を行う */
void rescue_end(bool flag)
{
	load_serial_ymode_close(flag);
}

/*  ymodeの受信する */
int	rescue_read(u8 *dest, int size)
{
	int ret = 0;

	ret = load_serial_ymodem(dest, size);

	return ret;
}

