/*
 * SuperH SCIF device driver.
 * Copyright (C) 2013  Renesas Electronics Corporation
 * Copyright (C) 2007,2008,2010 Nobuhiro Iwamatsu
 * Copyright (C) 2002 - 2008  Paul Mundt
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <asm/io.h>
#include <asm/processor.h>
#include "serial_sh.h"
#include <serial.h>
#include <linux/compiler.h>

#if defined(CONFIG_CONS_SCIF0)
# define SCIF_BASE	SCIF0_BASE
#elif defined(CONFIG_CONS_SCIF1)
# define SCIF_BASE	SCIF1_BASE
#elif defined(CONFIG_CONS_SCIF2)
# define SCIF_BASE	SCIF2_BASE
#elif defined(CONFIG_CONS_SCIF3)
# define SCIF_BASE	SCIF3_BASE
#elif defined(CONFIG_CONS_SCIF4)
# define SCIF_BASE	SCIF4_BASE
#elif defined(CONFIG_CONS_SCIF5)
# define SCIF_BASE	SCIF5_BASE
#elif defined(CONFIG_CONS_SCIF6)
# define SCIF_BASE	SCIF6_BASE
#elif defined(CONFIG_CONS_SCIF7)
# define SCIF_BASE	SCIF7_BASE
#else
# error "Default SCIF doesn't set....."
#endif

#if defined(CONFIG_SCIF_A)
	#define SCIF_BASE_PORT	PORT_SCIFA
#else
	#define SCIF_BASE_PORT	PORT_SCIF
#endif

static struct uart_port sh_sci0 = {
	.membase	= (unsigned char*)SCIF0_BASE,
	.mapbase	= SCIF0_BASE,
	.type		= SCIF_BASE_PORT,
};

static struct uart_port sh_sci2 = {
	.membase	= (unsigned char*)SCIF2_BASE,
	.mapbase	= SCIF2_BASE,
	.type		= SCIF_BASE_PORT,
};

static struct uart_port sh_sci;


static void sh_serial_setbrg0(void)
{
	DECLARE_GLOBAL_DATA_PTR;
#ifdef CONFIG_SCIF_USE_EXT_CLK
	unsigned short dl = DL_VALUE(gd->baudrate, CONFIG_SH_SCIF_CLK_FREQ);
	sci_out(&sh_sci0, DL, dl);
	/* Need wait: Clock * 1/dl $B!_(B 1/16 */
	udelay((1000000 * dl * 16 / CONFIG_SYS_CLK_FREQ) * 1000 + 1);
#else
	sci_out(&sh_sci0, SCBRR,
		SCBRR_VALUE(gd->baudrate, CONFIG_SH_SCIF_CLK_FREQ));
#endif
}

static void sh_serial_setbrg2(void)
{
	DECLARE_GLOBAL_DATA_PTR;
#ifdef CONFIG_SCIF_USE_EXT_CLK
	unsigned short dl = DL_VALUE(gd->baudrate, CONFIG_SH_SCIF_CLK_FREQ);
	sci_out(&sh_sci2, DL, dl);
	/* Need wait: Clock * 1/dl $B!_(B 1/16 */
	udelay((1000000 * dl * 16 / CONFIG_SYS_CLK_FREQ) * 1000 + 1);
#else
	sci_out(&sh_sci2, SCBRR,
		SCBRR_VALUE(gd->baudrate, CONFIG_SH_SCIF_CLK_FREQ));
#endif
}

static int sh_serial_init0(void)
{
	sci_out(&sh_sci0, SCSCR , SCSCR_INIT(&sh_sci0));
	sci_out(&sh_sci0, SCSCR , SCSCR_INIT(&sh_sci0));
	sci_out(&sh_sci0, SCSMR, 0);
	sci_out(&sh_sci0, SCSMR, 0);
	sci_out(&sh_sci0, SCFCR, SCFCR_RFRST|SCFCR_TFRST);
	sci_in(&sh_sci0, SCFCR);
	sci_out(&sh_sci0, SCFCR, 0);
#ifdef CONFIG_CPU_RZA1
	sci_out(&sh_sci0, SCSPTR, 0x0053);
#endif

	serial_setbrg();
	return 0;
}

static int sh_serial_init2(void)
{
	sci_out(&sh_sci2, SCSCR , SCSCR_INIT(&sh_sci2));
	sci_out(&sh_sci2, SCSCR , SCSCR_INIT(&sh_sci2));
	sci_out(&sh_sci2, SCSMR, 0);
	sci_out(&sh_sci2, SCSMR, 0);
	sci_out(&sh_sci2, SCFCR, SCFCR_RFRST|SCFCR_TFRST);
	sci_in(&sh_sci2, SCFCR);
	sci_out(&sh_sci2, SCFCR, 0);
#ifdef CONFIG_CPU_RZA1
	sci_out(&sh_sci2, SCSPTR, 0x0053);
#endif

	serial_setbrg();
	return 0;
}

#if defined(CONFIG_CPU_SH7760) || \
	defined(CONFIG_CPU_SH7780) || \
	defined(CONFIG_CPU_SH7785) || \
	defined(CONFIG_CPU_SH7786)
static int scif_rxfill(struct uart_port *port)
{
	return sci_in(port, SCRFDR) & 0xff;
}
#elif defined(CONFIG_CPU_SH7763)
static int scif_rxfill(struct uart_port *port)
{
	if ((port->mapbase == 0xffe00000) ||
		(port->mapbase == 0xffe08000)) {
		/* SCIF0/1*/
		return sci_in(port, SCRFDR) & 0xff;
	} else {
		/* SCIF2 */
		return sci_in(port, SCFDR) & SCIF2_RFDC_MASK;
	}
}
#elif defined(CONFIG_ARCH_SH7372)
static int scif_rxfill(struct uart_port *port)
{
	if (port->type == PORT_SCIFA)
		return sci_in(port, SCFDR) & SCIF_RFDC_MASK;
	else
		return sci_in(port, SCRFDR);
}
#else
static int scif_rxfill(struct uart_port *port)
{
	return sci_in(port, SCFDR) & SCIF_RFDC_MASK;
}
#endif

static int serial_rx_fifo_level0(void)
{
	return scif_rxfill(&sh_sci0);
}

static int serial_rx_fifo_level2(void)
{
	return scif_rxfill(&sh_sci2);
}

static void handle_error0(void)
{
	sci_in(&sh_sci0, SCxSR);
	sci_out(&sh_sci0, SCxSR, SCxSR_ERROR_CLEAR(&sh_sci0));
	sci_in(&sh_sci0, SCLSR);
	sci_out(&sh_sci0, SCLSR, 0x00);
}

static void handle_error2(void)
{
	sci_in(&sh_sci2, SCxSR);
	sci_out(&sh_sci2, SCxSR, SCxSR_ERROR_CLEAR(&sh_sci2));
	sci_in(&sh_sci2, SCLSR);
	sci_out(&sh_sci2, SCLSR, 0x00);
}


static void serial_raw_putc0(const char c)
{
	while (1) {
		/* Tx fifo is empty */
		if (sci_in(&sh_sci0, SCxSR) & SCxSR_TEND(&sh_sci0))
			break;
	}

	sci_out(&sh_sci0, SCxTDR, c);
	sci_out(&sh_sci0, SCxSR, sci_in(&sh_sci0, SCxSR) & ~SCxSR_TEND(&sh_sci0));
}

static void serial_raw_putc2(const char c)
{
	while (1) {
		/* Tx fifo is empty */
		if (sci_in(&sh_sci2, SCxSR) & SCxSR_TEND(&sh_sci2))
			break;
	}

	sci_out(&sh_sci2, SCxTDR, c);
	sci_out(&sh_sci2, SCxSR, sci_in(&sh_sci2, SCxSR) & ~SCxSR_TEND(&sh_sci2));
}

static void sh_serial_putc0(const char c)
{
	if (c == '\n')
		serial_raw_putc0('\r');
	serial_raw_putc0(c);
}

static void sh_serial_putc2(const char c)
{
	if (c == '\n')
		serial_raw_putc2('\r');
	serial_raw_putc2(c);
}

static int sh_serial_tstc0(void)
{
	if (sci_in(&sh_sci0, SCxSR) & SCIF_ERRORS) {
		handle_error0();
		return 0;
	}

	return serial_rx_fifo_level0() ? 1 : 0;
}

static int sh_serial_tstc2(void)
{
	if (sci_in(&sh_sci2, SCxSR) & SCIF_ERRORS) {
		handle_error2();
		return 0;
	}

	return serial_rx_fifo_level2() ? 1 : 0;
}


static int serial_getc_check0(void)
{
	unsigned short status;

	status = sci_in(&sh_sci0, SCxSR);

	if (status & SCIF_ERRORS)
		handle_error0();
	if (sci_in(&sh_sci, SCLSR) & SCxSR_ORER(&sh_sci))
		handle_error0();
	return status & (SCIF_DR | SCxSR_RDxF(&sh_sci));
}

static int serial_getc_check2(void)
{
	unsigned short status;

	status = sci_in(&sh_sci2, SCxSR);

	if (status & SCIF_ERRORS)
		handle_error2();
	if (sci_in(&sh_sci2, SCLSR) & SCxSR_ORER(&sh_sci2))
		handle_error2();
	return status & (SCIF_DR | SCxSR_RDxF(&sh_sci2));
}

static int sh_serial_getc0(void)
{
	unsigned short status;
	char ch;

	while (!serial_getc_check0())
		;

	ch = sci_in(&sh_sci0, SCxRDR);
	status = sci_in(&sh_sci0, SCxSR);

	sci_out(&sh_sci0, SCxSR, SCxSR_RDxF_CLEAR(&sh_sci0));

	if (status & SCIF_ERRORS)
			handle_error0();

	if (sci_in(&sh_sci0, SCLSR) & SCxSR_ORER(&sh_sci0))
		handle_error0();
	return ch;
}

static int sh_serial_getc2(void)
{
	unsigned short status;
	char ch;

	while (!serial_getc_check2())
		;

	ch = sci_in(&sh_sci2, SCxRDR);
	status = sci_in(&sh_sci2, SCxSR);

	sci_out(&sh_sci2, SCxSR, SCxSR_RDxF_CLEAR(&sh_sci2));

	if (status & SCIF_ERRORS)
			handle_error2();

	if (sci_in(&sh_sci2, SCLSR) & SCxSR_ORER(&sh_sci2))
		handle_error2();
	return ch;
}


static struct serial_device sh_serial_drv0 = {
	.name	= "sh_serial0",
	.start	= sh_serial_init0,
	.stop	= NULL,
	.setbrg	= sh_serial_setbrg0,
	.putc	= sh_serial_putc0,
	.puts	= default_serial_puts,
	.getc	= sh_serial_getc0,
	.tstc	= sh_serial_tstc0,
};

static struct serial_device sh_serial_drv2 = {
	.name	= "sh_serial2",
	.start	= sh_serial_init2,
	.stop	= NULL,
	.setbrg	= sh_serial_setbrg2,
	.putc	= sh_serial_putc2,
	.puts	= default_serial_puts,
	.getc	= sh_serial_getc2,
	.tstc	= sh_serial_tstc2,
};

void sh_serial_initialize(void)
{
	serial_register(&sh_serial_drv0);
	serial_register(&sh_serial_drv2);
}

#define MANUFACTURE_MAGIC	"magicword"
#define MAGIC_LENGTH		9
static int is_mfc_mode(void)
{
    char data[MAGIC_LENGTH] = {0};

    memcpy(data, (u8 *)0x18100000, MAGIC_LENGTH);
    if (memcmp(data, MANUFACTURE_MAGIC, MAGIC_LENGTH) == 0) {
        return 1;
    }
    return 0;
}

#define DCT1_ADDR				(0x18140000)
#define DCT2_ADDR				(0x18180000)
	#define ACTIVE_OFFSET		(4+4)
	#define BOOT_STATE_OFFSET	(32+4)
#define RESCUE_HALT_BIT			(0x40)
#define RESCUE_BIT				(0x10)
static int is_rescue_dct(void)
{
	u32 actv1 = 0, actv2 = 0;
	u32 mode = 0;

	memcpy(&actv1, (u32 *)(DCT1_ADDR+ACTIVE_OFFSET), 4);
	memcpy(&actv2, (u32 *)(DCT2_ADDR+ACTIVE_OFFSET), 4);
	if (((actv1&0x0000ff00) >> 8) & 0xff) {
		memcpy(&mode, (u32 *)(DCT1_ADDR+BOOT_STATE_OFFSET), 4);
	} else if (((actv2&0x0000ff00) >> 8) & 0xff) {
		memcpy(&mode, (u32 *)(DCT2_ADDR+BOOT_STATE_OFFSET), 4);
	}
	mode &= 0xff;
	if ((mode != 0xff) && (mode & (RESCUE_HALT_BIT|RESCUE_BIT))) {
		return 1;
	}
	return 0;
}

extern int is_rescue_sw(void);
static int is_rescue(void)
{
	return (is_rescue_sw() || is_rescue_dct());
}

__weak struct serial_device *default_serial_console(void)
{
	if (is_mfc_mode() || is_rescue()) {
		return &sh_serial_drv2;
    } else {
		return &sh_serial_drv0;
    }
}
